"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bootstrapShell = void 0;
const shell_registry_1 = require("./shell.registry");
const readline_1 = require("readline");
function bootstrapShell(options = { prompt: '⤳' }) {
    return __awaiter(this, void 0, void 0, function* () {
        const rl = readline_1.createInterface({
            input: process.stdin,
            output: process.stdout,
            prompt: `${options.prompt} `,
        });
        const print = options.shellPrinter || ((value) => console.log(value));
        const messages = Object.assign({ notFound: `Say what? I might have heard $input`, wrongUsage: `Wrong usage: $command $pattern` }, options.messages);
        const onLine = (line) => __awaiter(this, void 0, void 0, function* () {
            if (!line)
                return;
            const splittedLineResult = line.trim().split(' ');
            const command = shell_registry_1.ShellRegistry.findCommand(splittedLineResult[0]);
            if (!command) {
                print(messages.notFound.replace('$input', line));
                return;
            }
            const commandArguments = splittedLineResult.slice(1);
            return command.handler(commandArguments, { wrongUsage: messages.wrongUsage }, print);
        });
        rl.on('line', (input) => __awaiter(this, void 0, void 0, function* () { return onLine(input).then(() => rl.prompt()); })).on('close', () => process.exit());
        rl.emit('line');
    });
}
exports.bootstrapShell = bootstrapShell;
